
% REPRODUCING EXPERIMENT 1

clear; % we need to clear the variables in workspace before learning. 

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
epsilon = .1;  % allowed correlation between selected Gabors
T = 0.;        % threshold for responses
Totalsketch = 50;  % allowed number of selected Gabors   
Upperbound = 16.; % saturation value
sub = 1;    % subsampling rate for Gabor
Lrange = 6; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
%%% Note: Lrange*sub is the pixels allowed to shift along normal
%%%       Lrange*sub should be the same in learning and testing

LIK = 1.; SHUTUP= 1.;  % use log-likelihood score. If LIK=0, then use active correlation
% SHUTUP is to neglect the positions with low average responses
                       
% LIK = 0; SHUTUP = 1.;   


% TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
posFolder = 'e';  % image input folder
Iname = dir([posFolder '\*.jpg']);
nimage = size(Iname, 1);
Isize = zeros(nimage, 2);
for (i = 1 : nimage)
    I{i} = imresize_old(double(rgb2gray(imread([posFolder '\' Iname(i).name]))), .6);
    Ishow{i} = imread([posFolder '\' Iname(i).name]); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for (i = 1 : nimage)
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images the same


% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr(allfilter, epsilon);    % generate the inhibition maps
allfiltered = applyfilter(I, allfilter, T);  % filter training images


% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % average response of selected Gabor
sym = zeros(size(I{1}));  % symbolic plot of selected Gabor
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, 1);  % total scores for testing images
gain = zeros(Totalsketch, 1);  % coding gain for selected Gabor


binnum = 50; % number of bins 
% MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mex Clearn.c;   % compile C code
'learning'
tic
Clearn(nimage, norient, allfiltered, C, h, sx, sy, T, Mi, Mx, My, Mm, allsymbol(1, :), sym, Asym, Lrange, Orange, sub, Totalsketch, Upperbound, tss, gain, LIK, SHUTUP, binnum);
disp(['mex-C learning time: ' num2str(toc) ' seconds']);

% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
K = nimage;
[tsssort, ind] = sort(tss); 
figure; % display top K examples
col = 4; row = floor(K/col*2)+1; row = col;
subplot(row, col, 1, 'v6'); imshow(-(sym), []); 
for (i = 1 : K)
    subplot(row, col,  1+i*2, 'v6'); 
    imshow(Ishow{ind(nimage-i+1)}, []);
    subplot(row, col,  2+i*2, 'v6');
    imshow(-(Asym{ind(nimage-i+1)}), []); 
end

